import 'dart:io';

import 'package:flutter/material.dart';
import 'package:flutter/services.dart';
import 'package:flutter_keyboard_visibility/flutter_keyboard_visibility.dart';
import 'package:get/get.dart';
import 'package:intl/intl.dart';
import 'package:mirrorfly_plugin/flychat.dart';
import 'package:mirrorfly_plugin/model/callback.dart';
import 'package:mirrorfly_plugin/model/user_list_model.dart';
import 'package:mirrorfly_uikit_plugin/app/common/app_constants.dart';
import 'package:mirrorfly_uikit_plugin/app/common/extensions.dart';
import 'package:mirrorfly_uikit_plugin/mirrorfly_uikit.dart';
import 'package:mirrorfly_uikit_plugin/app/data/helper.dart';
import 'package:share_plus/share_plus.dart';

import '../../../common/constants.dart';
import '../../../model/chat_message_model.dart';

class StarredMessagesController extends FullLifeCycleController
    with FullLifeCycleMixin {
  var starredChatList = List<ChatMessageModel>.empty(growable: true).obs;
  double height = 0.0;
  double width = 0.0;
  var isSelected = false.obs;

  var isListLoading = true.obs;
  var calendar = DateTime.now();

  /* @override
  void onInit() {
    super.onInit();
    //getFavouriteMessages();
    player.onPlayerCompletion.listen((event) {
      playingChat!.mediaChatMessage!.isPlaying=false;
      playingChat!.mediaChatMessage!.currentPos=0;
      player.stop();
      starredChatList.refresh();
      playingChat=null;
    });

    player.onAudioPositionChanged.listen((Duration p) {
      playingChat?.mediaChatMessage!.currentPos=(p.inMilliseconds);
      starredChatList.refresh();
    });
  }*/

  /*@override
  void onClose(){
    super.onClose();
    // player.stop();
    // player.dispose();
  }*/
  @override
  void onHidden() {
    // Your implementation here
  }

  getFavouriteMessages() {
    if (!isSelected.value) {
      isListLoading(true);
      Mirrorfly.getFavouriteMessages().then((value) {
        if (value.toString().isNotEmpty) {
          List<ChatMessageModel> chatMessageModel =
              chatMessageModelFromJson(value);
          if (Platform.isAndroid) {
            starredChatList(chatMessageModel.toList());
          } else {
            var list = chatMessageModel.toList().reversed.toList();
            starredChatList(list);
          }
        }
        isListLoading(false);
        if (isSearch.value) {
          lastInputValue = Constants.emptyString;
          startSearch(searchedText.text.toString());
        }
      });
    }
  }

  void userUpdatedHisProfile(String jid) {
    if (jid.isNotEmpty) {
      getProfileDetails(jid).then((value) {
        var messageIndex =
            starredChatList.indexWhere((element) => element.chatUserJid == jid);
        if (!messageIndex.isNegative) {
          starredChatList.refresh();
        }
      });
    }
  }

  void onMessageStatusUpdated(chatMessageModel) {
    final index = starredChatList.indexWhere(
        (message) => message.messageId == chatMessageModel.messageId);
    debugPrint("Message Status Update index of search $index");
    if (!index.isNegative) {
      starredChatList[index].messageStatus = chatMessageModel.messageStatus;
      starredChatList.refresh();
    }
  }

  void onMediaStatusUpdated(ChatMessageModel chatMessageModel) {
    final index = starredChatList.indexWhere(
        (message) => message.messageId == chatMessageModel.messageId);
    if (!index.isNegative) {
      starredChatList[index].mediaChatMessage?.mediaLocalStoragePath(
          chatMessageModel.mediaChatMessage!.mediaLocalStoragePath.value);
      starredChatList[index].mediaChatMessage?.mediaDownloadStatus(
          chatMessageModel.mediaChatMessage!.mediaDownloadStatus.value);
      starredChatList[index].mediaChatMessage?.mediaUploadStatus(
          chatMessageModel.mediaChatMessage!.mediaUploadStatus.value);
    }

    if (isSelected.value) {
      var selectedIndex = selectedChatList.indexWhere(
          (element) => chatMessageModel.messageId == element.messageId);
      if (!selectedIndex.isNegative) {
        chatMessageModel
            .isSelected(true); //selectedChatList[selectedIndex].isSelected;
        selectedChatList[selectedIndex] = chatMessageModel;
        selectedChatList.refresh();
        validateForForwardMessage();
        validateForShareMessage();
      }
    }
    if (isSearch.value) {
      var selectedIndex = searchedStarredMessageList.indexWhere(
          (element) => chatMessageModel.messageId == element.messageId);
      if (!selectedIndex.isNegative) {
        searchedStarredMessageList[selectedIndex] = chatMessageModel;
      }
    }
  }

  void onUploadDownloadProgressChanged(
      String messageId, String progressPercentage) {
    if (messageId.isNotEmpty) {
      final index = starredChatList
          .indexWhere((message) => message.messageId == messageId);
      debugPrint(
          "Media Status Onprogress changed---> onUploadDownloadProgressChanged $index $messageId $progressPercentage");
      if (!index.isNegative) {
        starredChatList[index]
            .mediaChatMessage
            ?.mediaProgressStatus(int.parse(progressPercentage));
        starredChatList.refresh();
      }
      if (isSearch.value) {
        var selectedIndex = searchedStarredMessageList
            .indexWhere((message) => message.messageId == messageId);
        if (!selectedIndex.isNegative) {
          searchedStarredMessageList[selectedIndex]
              .mediaChatMessage
              ?.mediaProgressStatus(int.parse(progressPercentage));
        }
      }
    }
  }

  String getChatTime(int? epochTime) {
    if (epochTime == null) return Constants.emptyString;
    if (epochTime == 0) return Constants.emptyString;
    var convertedTime = epochTime; // / 1000;
    //messageDate.time = convertedTime
    // var hourTime = manipulateMessageTime(
    //     context, DateTime.fromMicrosecondsSinceEpoch(convertedTime));
    var currentYear = DateTime.now().year;
    calendar = DateTime.fromMicrosecondsSinceEpoch(convertedTime);
    var time = (currentYear == calendar.year)
        ? DateFormat("dd-MMM-yyyy").format(calendar)
        : DateFormat("yyyy/MM/dd").format(calendar);
    return time;
  }

  String manipulateMessageTime(context, DateTime messageDate) {
    var format = MediaQuery.of(context).alwaysUse24HourFormat ? 24 : 12;
    var hours = calendar.hour; //calendar[Calendar.HOUR]
    calendar = messageDate;
    var dateHourFormat = setDateHourFormat(format, hours);
    return DateFormat(dateHourFormat).format(messageDate);
  }

  String setDateHourFormat(int format, int hours) {
    var dateHourFormat = (format == 12)
        ? (hours < 10)
            ? "hh:mm aa"
            : "h:mm aa"
        : (hours < 10)
            ? "HH:mm"
            : "H:mm";
    return dateHourFormat;
  }

  var selectedChatList = List<ChatMessageModel>.empty(growable: true).obs;

  void addChatSelection(ChatMessageModel item) {
    if (item.messageType.toUpperCase() != Constants.mNotification) {
      selectedChatList.add(item);
      item.isSelected(true);
      starredChatList.refresh();
      validateForForwardMessage();
      validateForShareMessage();
    } else {
      debugPrint("Unable to Select Notification Banner");
    }
  }

  clearChatSelection(ChatMessageModel item) {
    selectedChatList.remove(item);
    item.isSelected(false);
    if (selectedChatList.isEmpty) {
      isSelected(false);
      selectedChatList.clear();
    }
    starredChatList.refresh();
    validateForForwardMessage();
    validateForShareMessage();
  }

  clearAllChatSelection() {
    isSelected(false);
    for (var chatItem in selectedChatList) {
      chatItem.isSelected(false);
    }
    selectedChatList.clear();
  }

  bool getOptionStatus(String optionName) {
    switch (optionName) {
      case 'Reply':
        return selectedChatList.length > 1 ? false : true;

      case 'Report':
        return selectedChatList.length > 1
            ? false
            : selectedChatList[0].isMessageSentByMe
                ? false
                : true;

      case 'Message Info':
        return selectedChatList.length > 1
            ? false
            : selectedChatList[0].isMessageSentByMe
                ? true
                : false;

      case 'Share':
        for (var chatList in selectedChatList) {
          if (chatList.messageType == Constants.mText ||
              chatList.messageType == Constants.mLocation ||
              chatList.messageType == Constants.mContact) {
            return false;
          }
        }
        return true;

      case 'Favourite':
        // for (var chatList in selectedChatList) {
        //   if (chatList.isMessageStarred) {
        //     return true;
        //   }
        // }
        // return false;
        return selectedChatList.length > 1 ? false : true;

      default:
        return false;
    }
  }

  checkBusyStatusForForward(BuildContext context) async {
    var busyStatus = await Mirrorfly.isBusyStatusEnabled();
    if (!busyStatus.checkNull()) {
      if (context.mounted) forwardMessage(context);
    } else {
      if (context.mounted) showBusyStatusAlert(forwardMessage, context);
    }
  }

  showBusyStatusAlert(Function? function, BuildContext context) {
    Helper.showAlert(
        message: AppConstants.disableBusy,
        actions: [
          TextButton(
              onPressed: () {
                // Get.back();
                Navigator.pop(context);
              },
              child: Text(
                AppConstants.no,
                style: TextStyle(color: MirrorflyUikit.getTheme?.primaryColor),
              )),
          TextButton(
              onPressed: () async {
                // Get.back();
                Navigator.pop(context);
                await Mirrorfly.enableDisableBusyStatus(
                    enable: false,
                    flyCallBack: (FlyResponse response) {
                      if (response.isSuccess) {
                        if (function != null) {
                          function();
                        }
                      }
                    });
              },
              child: Text(
                AppConstants.yes,
                style: TextStyle(color: MirrorflyUikit.getTheme?.primaryColor),
              )),
        ],
        context: context);
  }

  forwardMessage(BuildContext context) {
    var messageIds = List<String>.empty(growable: true);
    for (var chatItem in selectedChatList) {
      messageIds.add(chatItem.messageId);
      debugPrint(messageIds.length.toString());
      debugPrint(selectedChatList.length.toString());
    }
    if (messageIds.length == selectedChatList.length) {
      isSelected(false);
      selectedChatList.clear();
      // Get.toNamed(Routes.forwardChat, arguments: {
      //   "forward": true,
      //   "group": false,
      //   "groupJid": "",
      //   "messageIds": messageIds
      // })?.then((value) {
      //   if (value != null) {
      //     debugPrint(
      //         "result of forward ==> ${(value as Profile).toJson().toString()}");
      //
      //     Get.toNamed(Routes.chat, arguments: value);
      //   }
      // });

      Navigator.push(
              context,
              MaterialPageRoute(
                  builder: (con) =>
                      ForwardChatView(forwardMessageIds: messageIds)))
          .then((value) {
        if (value != null) {
          debugPrint(
              "result of forward ==> ${(value as ProfileDetails).toJson().toString()}");
          Navigator.push(context,
              MaterialPageRoute(builder: (con) => ChatView(jid: value.jid!)));
        }
      });
    }
  }

  favouriteMessage() {
    for (var item in selectedChatList) {
      Mirrorfly.updateFavouriteStatus(
          messageId: item.messageId,
          chatUserJid: item.chatUserJid,
          isFavourite: !item.isMessageStarred.value,
          chatType: item.messageChatType,
          flyCallBack: (FlyResponse response) {});
      starredChatList
          .removeWhere((element) => item.messageId == element.messageId);
      if (isSearch.value) {
        searchedStarredMessageList
            .removeWhere((element) => item.messageId == element.messageId);
      }
    }
    selectedChatList.clear();
    isSelected(false);
  }

  copyTextMessages() {
    Clipboard.setData(
        ClipboardData(text: selectedChatList[0].messageTextContent.toString()));
    clearChatSelection(selectedChatList[0]);
    toToast(AppConstants.textCopied);
  }

  Map<bool, bool> isMessageCanbeRecalled() {
    var recallTimeDifference =
        ((DateTime.now().millisecondsSinceEpoch - 30000) * 1000);
    return {
      selectedChatList.any((element) =>
              element.isMessageSentByMe &&
              !element.isMessageRecalled.value &&
              (element.messageSentTime > recallTimeDifference)):
          selectedChatList.any((element) =>
              !element.isMessageRecalled.value &&
              (element.isMediaMessage() &&
                  element.mediaChatMessage!.mediaLocalStoragePath.value
                      .checkNull()
                      .isNotEmpty))
    };
  }

  void deleteMessages(BuildContext context) {
    //var isRecallAvailable = isMessageCanbeRecalled().keys.first;
    var isCheckBoxShown = isMessageCanbeRecalled().values.first;
    /*var deleteChatListID = List<String>.empty(growable: true);
    for (var element in selectedChatList) {
      deleteChatListID.add(element.messageId);
    }
    if (deleteChatListID.isEmpty) {
      return;
    }*/
    var isMediaDelete = false.obs;
    //var chatType =  profile.isGroupProfile ?? false ? "groupchat" : "chat";
    Helper.showAlert(
        content: Column(
          mainAxisSize: MainAxisSize.min,
          children: [
            Text(
              selectedChatList.length > 1
                  ? AppConstants.deleteSelectedMessages
                  : AppConstants.deleteSelectedMessage,
              // "Are you sure you want to delete selected Message${selectedChatList.length > 1 ? "s" : ""}?"
              style:
                  TextStyle(color: MirrorflyUikit.getTheme?.textSecondaryColor),
            ),
            isCheckBoxShown
                ? Column(
                    mainAxisSize: MainAxisSize.min,
                    children: [
                      InkWell(
                        onTap: () {
                          isMediaDelete(!isMediaDelete.value);
                          mirrorFlyLog(
                              "isMediaDelete", isMediaDelete.value.toString());
                        },
                        child: Row(
                          children: [
                            Obx(() {
                              return Theme(
                                data: ThemeData(
                                  unselectedWidgetColor: Colors.grey,
                                ),
                                child: Checkbox(
                                    value: isMediaDelete.value,
                                    activeColor: MirrorflyUikit
                                        .getTheme!.primaryColor, //Colors.white,
                                    checkColor:
                                        MirrorflyUikit.getTheme?.colorOnPrimary,
                                    onChanged: (value) {
                                      isMediaDelete(!isMediaDelete.value);
                                      mirrorFlyLog(
                                          "isMediaDelete", value.toString());
                                    }),
                              );
                            }),
                            Expanded(
                              child: Text(AppConstants.deleteMediaFromPhone,
                                  style: TextStyle(
                                      color: MirrorflyUikit
                                          .getTheme?.textSecondaryColor)),
                            ),
                          ],
                        ),
                      )
                    ],
                  )
                : const SizedBox(),
          ],
        ),
        message: Constants.emptyString,
        actions: [
          TextButton(
              onPressed: () {
                // Get.back();
                Navigator.pop(context);
              },
              child: Text(AppConstants.cancel.toUpperCase(),
                  style:
                      TextStyle(color: MirrorflyUikit.getTheme?.primaryColor))),
          TextButton(
              onPressed: () {
                // Get.back();
                Navigator.pop(context);
                // for (var item in selectedChatList) {
                var messageIds =
                    selectedChatList.map((item) => item.messageId).toList();
                Mirrorfly.deleteMessagesForMe(
                    jid: selectedChatList[0].chatUserJid,
                    chatType: selectedChatList[0].messageChatType,
                    messageIds: messageIds,
                    isMediaDelete: isMediaDelete.value,
                    flyCallBack: (FlyResponse response) {
                      for (var item in messageIds) {
                        starredChatList.removeWhere(
                            (element) => item == element.messageId);
                        if (isSearch.value) {
                          searchedStarredMessageList.removeWhere(
                              (element) => item == element.messageId);
                        }
                      }
                      isSelected(false);
                      selectedChatList.clear();
                    });

                /* Mirrorfly.deleteMessagesForMe(
                      item.chatUserJid,
                      item.messageChatType,
                      [item.messageId],
                      isMediaDelete.value);
                  starredChatList.removeWhere(
                      (element) => item.messageId == element.messageId);
                  if(isSearch.value){
                    searchedStarredMessageList
                        .removeWhere((element) => item.messageId == element.messageId);
                  }*/
                // }
                isSelected(false);
                selectedChatList.clear();
              },
              child: Text(AppConstants.deleteForMe.toUpperCase(),
                  style:
                      TextStyle(color: MirrorflyUikit.getTheme?.primaryColor))),
          /*isRecallAvailable
              ? TextButton(
              onPressed: () {
                Get.back();
                Helper.showLoading(
                    message: 'Deleting Message for Everyone');
                */ /*Mirrorfly.deleteMessagesForEveryone(
                    profile.jid!,chatType, deleteChatListID, isMediaDelete.value)
                    .then((value) {
                  debugPrint(value.toString());
                  Helper.hideLoading();
                  if (value!=null && value) {
                    // removeChatList(selectedChatList);//
                    for (var chatList in selectedChatList) {
                      chatList.isMessageRecalled = true;
                      chatList.isSelected=false;
                      this.chatList.refresh();
                    }
                  }
                  isSelected(false);
                  selectedChatList.clear();
                });*/ /*
              },
              child: const Text("DELETE FOR EVERYONE"))
              : const SizedBox.shrink(),*/
        ],
        context: context);
  }

  // AudioPlayer player = AudioPlayer();
  ChatMessageModel? playingChat;
  playAudio(ChatMessageModel chatMessage) async {
    /*setPlayingChat(chatMessage);
    if (!playingChat!.mediaChatMessage!.isPlaying) {
      int result = await player.play(playingChat!.mediaChatMessage!.mediaLocalStoragePath,position: Duration(milliseconds:playingChat!.mediaChatMessage!.currentPos), isLocal: true);
      if (result == 1) {
        playingChat!.mediaChatMessage!.isPlaying=true;
      } else {
        mirrorFlyLog("", "Error while playing audio.");
      }
    } else if (!playingChat!.mediaChatMessage!.isPlaying) {
      int result = await player.resume();
      if (result == 1) {
        playingChat!.mediaChatMessage!.isPlaying=true;
        starredChatList.refresh();
      } else {
        mirrorFlyLog("", "Error on resume audio.");
      }
    } else {
      int result = await player.pause();
      if (result == 1) {
        playingChat!.mediaChatMessage!.isPlaying=false;
        starredChatList.refresh();
      } else {
        mirrorFlyLog("", "Error on pause audio.");
      }
    }*/
  }

  void setPlayingChat(ChatMessageModel chatMessage) {
    /*if(playingChat!=null){
      if(playingChat?.mediaChatMessage!.messageId!=chatMessage.messageId){
        player.stop();
        playingChat?.mediaChatMessage!.isPlaying=false;
        playingChat = chatMessage;
      }
    }
    else{
      playingChat = chatMessage;
    }*/
  }
  void onSeekbarChange(double value, ChatMessageModel chatMessage) {
    /* debugPrint('onSeekbarChange $value');
    if (playingChat != null) {
      player.seek(Duration(milliseconds: value.toInt()));
    }else{
      chatMessage.mediaChatMessage?.currentPos=value.toInt();
      // starredChatList.refresh();
    }*/
  }

  RxBool canBeForward = false.obs;
  validateForForwardMessage() {
    for (var value in selectedChatList) {
      if (value.isMediaMessage()) {
        if ((value.isMediaDownloaded() || value.isMediaUploaded()) &&
            value.mediaChatMessage!.mediaLocalStoragePath.value
                .checkNull()
                .isNotEmpty) {
          canBeForward(true);
        } else {
          canBeForward(false);
          break;
        }
      } else {
        canBeForward(true);
      }
    }
  }

  RxBool canBeShare = false.obs;
  validateForShareMessage() {
    for (var value in selectedChatList) {
      if (value.isMediaMessage()) {
        if ((value.isMediaDownloaded() || value.isMediaUploaded()) &&
            checkFile(value.mediaChatMessage!.mediaLocalStoragePath.value
                .checkNull())) {
          canBeShare(true);
        } else {
          canBeShare(false);
          break;
        }
      } else {
        canBeShare(false);
        break;
      }
    }
  }

  var isSearch = false.obs;
  var clear = false.obs;
  var searchedText = TextEditingController();
  String lastInputValue = Constants.emptyString;
  void startSearch(String str) {
    if (str.isNotEmpty) {
      clear(true);
      if (lastInputValue != str.trim()) {
        starredChatList.clear();
        lastInputValue = str.trim();
        addSearchedMessagesToList(str.trim());
      }
    } else {
      lastInputValue = Constants.emptyString;
      clear(false);
      starredChatList.clear();
      starredChatList.addAll(searchedStarredMessageList);
      starredChatList.refresh();
    }
  }

  onSearchClick() {
    if (isSearch.value) {
      isSearch(false);
    } else {
      isSearch(true);
      searchedStarredMessageList.clear();
      searchedStarredMessageList.addAll(starredChatList);
    }
  }

  clearSearch() {
    lastInputValue = Constants.emptyString;
    isSearch(false);
    clear(false);
    searchedText.clear();
    starredChatList.clear();
    starredChatList.addAll(searchedStarredMessageList);
    searchedStarredMessageList.clear();
    starredChatList.refresh();
  }

  var searchedStarredMessageList = <ChatMessageModel>[];
  Future<void> addSearchedMessagesToList(String filterKey) async {
    if (starredChatList.isEmpty) {
      for (var message in searchedStarredMessageList) {
        if (isTextMessageContainsFilterKey(message, filterKey)) {
          if (starredChatList
              .indexWhere((element) => element.messageId == message.messageId)
              .isNegative) {
            starredChatList.add(message);
            // debugPrint('starredChatList ${message.messageId}text');
          }
        } else if (isImageCaptionContainsFilterKey(message, filterKey)) {
          if (starredChatList
              .indexWhere((element) => element.messageId == message.messageId)
              .isNegative) {
            starredChatList.add(message);
            // debugPrint('starredChatList ${message.messageId}image');
          }
        } else if (isVideoCaptionContainsFilterKey(message, filterKey)) {
          if (starredChatList
              .indexWhere((element) => element.messageId == message.messageId)
              .isNegative) {
            starredChatList.add(message);
            // debugPrint('starredChatList ${message.messageId}video');
          }
        } else if (Constants.mDocument == message.messageType &&
            message.mediaChatMessage!.mediaFileName.checkNull().isNotEmpty &&
            message.mediaChatMessage!.mediaFileName
                .toLowerCase()
                .contains(filterKey.toLowerCase())) {
          if (starredChatList
              .indexWhere((element) => element.messageId == message.messageId)
              .isNegative) {
            starredChatList.add(message);
            // debugPrint('starredChatList ${message.messageId}doc');
          }
        } else if (Constants.mContact == message.messageType &&
            message.contactChatMessage!.contactName.checkNull().isNotEmpty &&
            message.contactChatMessage!.contactName
                .toLowerCase()
                .contains(filterKey.toLowerCase())) {
          if (starredChatList
              .indexWhere((element) => element.messageId == message.messageId)
              .isNegative) {
            starredChatList.add(message);
            // debugPrint('starredChatList ${message.messageId}contact');
          }
        } else if (message.senderUserName.checkNull().isNotEmpty &&
            message.senderUserName
                .toLowerCase()
                .contains(filterKey.toLowerCase())) {
          if (starredChatList
              .indexWhere((element) => element.messageId == message.messageId)
              .isNegative) {
            starredChatList.add(message);
            // debugPrint('starredChatList ${message.messageId}sender');
          }
        } else if (message.isMessageSentByMe &&
            AppConstants.you.toLowerCase().contains(filterKey.toLowerCase())) {
          if (starredChatList
              .indexWhere((element) => element.messageId == message.messageId)
              .isNegative) {
            starredChatList.add(message);
            // debugPrint('starredChatList ${message.messageId}you');
          }
        } else if ((message.messageChatType == Constants.typeGroupChat)) {
          var name = await getProfileDetails(message.chatUserJid.checkNull());
          if (name.name.checkNull().contains(filterKey.toLowerCase())) {
            if (starredChatList
                .indexWhere((element) => element.messageId == message.messageId)
                .isNegative) {
              starredChatList.add(message);
              // debugPrint('starredChatList ${message.messageId} : group');
            }
          }
        }
        starredChatList.refresh();
      }
    } else {
      starredChatList([]);
      addSearchedMessagesToList(filterKey);
    }
    debugPrint('starredChatList ${starredChatList.length}');
    /*starredMessagesAdapterAdapterData!!.setSearch(searchEnabled, searchedText)
  starredMessagesAdapterAdapterData!!.setStarredMessages(searchedStarredMessageList)
  starredMessagesAdapterAdapterData!!.notifyDataSetChanged()*/
  }

  bool isVideoCaptionContainsFilterKey(
      ChatMessageModel message, String filterKey) {
    return Constants.mVideo == message.messageType &&
        message.mediaChatMessage!.mediaCaptionText.checkNull().isNotEmpty &&
        message.mediaChatMessage!.mediaCaptionText
            .checkNull()
            .toLowerCase()
            .contains(filterKey.toLowerCase());
  }

  bool isImageCaptionContainsFilterKey(
      ChatMessageModel message, String filterKey) {
    return Constants.mImage == message.messageType &&
        message.mediaChatMessage!.mediaCaptionText.checkNull().isNotEmpty &&
        message.mediaChatMessage!.mediaCaptionText
            .checkNull()
            .toLowerCase()
            .contains(filterKey.toLowerCase());
  }

  bool isTextMessageContainsFilterKey(
      ChatMessageModel message, String filterKey) {
    return Constants.mText == message.messageType &&
        message.messageTextContent
            .checkNull()
            .toLowerCase()
            .contains(filterKey.toLowerCase());
  }
  /* Future<Profile> getProfile(String jid) async {
    var value = await Mirrorfly.getProfileDetails(jid);
    return Profile.fromJson(json.decode(value.toString()));
  }*/

  navigateMessage(ChatMessageModel starredChat, BuildContext context) {
    Navigator.push(
        context,
        MaterialPageRoute(
            builder: (con) => ChatView(
                  jid: starredChat.chatUserJid,
                  isFromStarred: true,
                  messageId: starredChat.messageId,
                )));
    // Get.toNamed(Routes.chat,parameters: {'isFromStarred':'true',"userJid":starredChat.chatUserJid,"messageId":starredChat.messageId});
  }

  void share() {
    var mediaPaths = <XFile>[];
    for (var item in selectedChatList) {
      if (item.isMediaMessage()) {
        if ((item.isMediaDownloaded() || item.isMediaUploaded()) &&
            item.mediaChatMessage!.mediaLocalStoragePath.value
                .checkNull()
                .isNotEmpty) {
          mediaPaths.add(XFile(
              item.mediaChatMessage!.mediaLocalStoragePath.value.checkNull()));
        }
      }
    }
    clearAllChatSelection();
    Share.shareXFiles(mediaPaths);
  }

  void userDeletedHisProfile(String jid) {
    userUpdatedHisProfile(jid);
  }

  @override
  void onDetached() {}

  @override
  void onInactive() {}

  @override
  void onPaused() {}

  FocusNode searchFocus = FocusNode();
  @override
  void onResumed() {
    if (isSearch.value) {
      if (!KeyboardVisibilityController().isVisible) {
        if (searchFocus.hasFocus) {
          searchFocus.unfocus();
          Future.delayed(const Duration(milliseconds: 100), () {
            searchFocus.requestFocus();
          });
        }
      }
    }
  }
}
